@@ -1025,6 +1025,14 @@ class Column:
                     if isinstance(v,(list,str)):
                         self.add_form('delete',v)
 
+                if self.create_form is True and self.no_update is False:
+                    self.update_form = True
+
+                if self._column.data.is_hash_id is True or self._column.data.is_primary is True:
+                    self.delete_form = True
+                    self.no_delete = False
+                    self.no_post_args = False
+
                 if hasattr(self,k):
                     setattr(self,k,v)
                 else:
