@@ -11,7 +11,7 @@ from typing import Iterable, Union
 
 
 
-from colemen_config import _db_column_type,_db_mysql_database_type,_db_mysql_manager_type,_db_relationship_type
+from colemen_config import _db_column_type,_db_mysql_database_type,_db_table_type,_db_relationship_type
 # import colemen_utilities.database_utils.MySQL.Column.Column as _Column
 from colemen_utilities.database_utils.MySQL.Column.Column import Column as _Column
 # from colemen_utilities.database_utils.MySQL.Column import column_utils as _u
@@ -60,14 +60,20 @@ class Table:
     
     # ----------------------------------- ROLES ---------------------------------- #
     create_roles = None
+    '''The roles allowed to insert to this table'''
     read_roles = None
+    '''The roles allowed to read from this table'''
     update_roles = None
+    '''The roles allowed to update this table'''
     delete_roles = None
+    '''The roles allowed to delete rows in this table'''
     
     
     # create_time = None
     # update_time = None
     # check_time = None
+    subject_type:str = None
+    '''The type of subject this table represents [entity,relationship]'''
     table_collation = None
     checksum = None
     create_options = None
@@ -87,6 +93,11 @@ class Table:
     '''A dictionary of relationship instances that are associated to this table.'''
 
 
+    _column_names = None
+    '''Contains a list of all column names in this table.'''
+    _longest_column_name = None
+    '''Contains the longest column name tuple'''
+
     raw_fulltext_indexes= None
     '''Contains a list of dictionaries, each representing a column that has a fulltext index in this table.'''
     raw_referential_constraints= None
@@ -234,6 +245,7 @@ class Table:
             "read_roles":self.read_roles,
             "update_roles":self.update_roles,
             "delete_roles":self.delete_roles,
+            "subject_type":self.subject_type,
             # "create_options":self.create_options,
             # "create_roles":self.create_roles,
             # "read_roles":self.read_roles,
@@ -393,6 +405,46 @@ class Table:
                 return f"{self.hash_id_prefix}_{_rand.rand((self._hash_id_column.data.character_maximum_length - len(self.hash_id_prefix)))}"
 
 
+    @property
+    def child_tables(self)->Iterable[_db_table_type]:
+        '''
+            Get all table instances that are children of this table.
+
+            `default`:None
+
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-21-2022 09:49:13
+            `@memberOf`: Table
+            `@property`: child_tables
+        '''
+        value = self.database.get_table_children(self.name)
+        return value
+
+    @property
+    def parent_tables(self)->Iterable[_db_table_type]:
+        '''
+            Get all table instances that are parents of this table.
+
+            `default`:None
+
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-21-2022 09:49:13
+            `@memberOf`: Table
+            `@property`: child_tables
+        '''
+        value = self.database.get_table_parents(self.name)
+        return value
+
+
+
+
+
     # ---------------------------------------------------------------------------- #
     #                            COLUMN SUPPORT METHODS                            #
     # ---------------------------------------------------------------------------- #
@@ -466,6 +518,25 @@ class Table:
                     break
         return value
     
+    @property
+    def primary_column_name(self):
+        '''
+            Get this Table's primary column name
+
+            `default`:None
+
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-21-2022 08:35:46
+            `@memberOf`: Table
+            `@property`: primary_column_name
+        '''
+        value = self.primary_id.data.column_name
+        return value
+    
+    
     @property
     def hash_id(self):
         '''
@@ -747,11 +818,48 @@ class Table:
         '''
         self._has_modified_timestamp_column = value
 
-    # def locate_special_columns(self):
-    #     for col in self.columns:
-    #         if col.data.is_hash_id:
-    #             self.has_hash_id = True
+    @property
+    def column_names(self):
+        '''
+            Get a list of this Table's column names
 
+            `default`:None
+
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-21-2022 08:38:55
+            `@memberOf`: Table
+            `@property`: column_names
+        '''
+        if self._column_names is None:
+            names = []
+            for col in self.columns:
+                names.append(col.data.column_name)
+            self._column_names = names
+        return self._column_names
+
+    @property
+    def longest_column_name(self):
+        '''
+            Get this Table's longest column name as a tuple:
+
+            (20,someFuckinColumnName)
+
+            `default`:None
+
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-21-2022 08:37:11
+            `@memberOf`: Table
+            `@property`: longest_column_name
+        '''
+        if self._longest_column_name is None:
+            self._longest_column_name = _lu.longest_string(self.column_names)
+        return self._longest_column_name
 
 
 
@@ -862,7 +970,61 @@ class Table:
     def create_cerberus_validation_schema(self,**kwargs):
         _obj.get_kwarg(['post_arguments'],False,)
 
-        
+    @property
+    def comment(self)->str:
+        '''
+            Get this tables comment if there is one.
+
+            `default`:None
+
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-20-2022 11:18:08
+            `@memberOf`: PostArg
+            `@property`: comment
+        '''
+        value = self.table_comment
+        return value
+
+    @comment.setter
+    def comment(self,value:str):
+        '''
+            Set this tables comment.
+
+            This will execute an ALTER query to save the comment to the table.
+
+            Meta
+            ----------
+            `@author`: Colemen Atwood
+            `@created`: 12-20-2022 11:18:08
+            `@memberOf`: PostArg
+            `@property`: comment
+        '''
+        self.table_comment = value
+        sql = f"ALTER TABLE `{self.database.name}`.`{self.name}` COMMENT = '%s';"
+        args = [self.table_comment]
+        return self.database.run(sql,args)
+
+
+
+
+
+
+
+
+    def api_endpoint_url(self,crud_type:str=None):
+        if crud_type is None or crud_type in ['create','update','delete','read_single']:
+            return self.api_url_base
+        if crud_type in ['read']:
+            return f"/{self.name_.camel.plural}"
+    @property
+    def api_url_base(self):
+        if self.has_hash_id:
+            return f"/{self.name_.camel.singular}/{self._hash_id_column.data.flask_url_placeholder}"
+        return f"/{self.name_.camel.singular}"
+
 
 
 
@@ -905,6 +1067,7 @@ def populate_from_dict(data:dict,table:Table):
 
         # @Mstep [IF] if the key is table_comment
         if k == "table_comment":
+            _log(f"table.populate.from_dict - table_comment found {table.name}","magenta invert")
             # @Mstep [] parse the comment as YAML
             yd = parse_comment_yaml(v)
             if isinstance(yd,(dict)):
